#!/usr/bin/env rexx

--parse arg gladeFile
guiFile = "/usr/local/gui/radio.ui"
pipeFile = "~/.local/share/internetradio/fifo"
radioDir = "~/.local/share/internetradio"
radioFile = radioDir"/radio.cfg"
helpFile = "/usr/local/bin/internetRadio.hlp"

-- check existance or create ~/.local/share/intenetradio
-- for storing the list of radio stations
address system 'mkdir -p' radioDir

-- initialize RexxGTK
call gtk_init

-- check musicplayer availability
music_player = "mpg123"
if (check_executable_available(music_player)=.false) then exit

-- check availability of url scanner
urlscanner = "wget"
if (check_executable_available(urlscanner)=.false) then exit

--start the music player
if (start_music_player(pipeFile, music_player)=.false) then exit

-- build the user interface from a Glade file
builder = .GtkBuilder~new_from_file(guiFile)

-- get access to the top level window.
window = .mainWindow~from_builder(builder,"WINDOW")
window~set_title("RexxGTK - MP3 Internet Radio Player")

--get access to the scrolled window
scrolledWindow = .GtkScrolledWindoW~from_builder(builder,"STATIONS")

-- load the array with saved radio stations from .radio.cfg
window~radioFile = radioFile
call load_stations window

-- create the Treeview to be used for the stations
window~stationView = .stationView~new

-- create the Treeview columns (i.e. station's name and url
call setup_tree_view window~stationView, window~COL_NAME, window~COL_URL

-- set selection to single row
selection = window~stationView~get_selection
selection~set_mode(.gtk~GTK_SELECTION_SINGLE);

--selection~connect_signal("changed")
window~selection = selection

-- create the Liststore for the radio stations
store = .GtkListStore~new(.gtk~G_TYPE_STRING, .gtk~G_TYPE_STRING)
window~listStore = store

-- fill the listStore wih the stations
call add_stations_to_list window

-- set the Liststore as the model for the radioView
window~stationView~set_model(store)
window~stationView~user_data = window

-- add the stationView to the scrolled window
scrolledWindow~add(window~stationView)

-- get access to the quitButton to exit the program
quitButton = .quitButton~from_builder(builder, "QUIT")

-- get access to the aboutButton provide some info
aboutButton = .aboutButton~from_builder(builder, "ABOUT")

--get access to the helpButton to get some instructions
helpButton = .helpButton~from_builder(builder, "README")

-- get access to the playButton to stream a radiostation
playButton = .playButton~from_builder(builder, "PLAY")

-- get access to the pauseButton to stop streaming
pauseButton = .pauseButton~from_builder(builder, "PAUSE")

-- get access to addButton to add a new radiostation
addButton = .addButton~from_builder(builder, "ADD")
addButton~user_data = window

-- get access to the editButton to change a radiostation
editButton = .editButton~from_builder(builder, "EDIT")
editButton~user_data = window

-- get access to the deleteButton to delete a radiostation
deleteButton = .deleteButton~from_builder(builder, "DEL")

-- get access to the titleLabel to display music titles
streamLabel = .gtkLabel~from_builder(builder,"LABEL")
streamLabel~set_label("Hello")
window~streamLabel = streamLabel

-- start the ticker to update streamTitle if streaming is active
window~streamTicker = .Ticker~new(10,.streamScanner~new,window)

-- get access to add an internet station dialog
window~radioDialog = .GtkDialog~from_builder(builder,"CONFIG")

-- get access the entries for a radiostation
window~nameEntry = .GtkEntry~from_builder(builder,"ADD_NAME")
window~urlEntry = .GtkEntry~from_builder(builder,"ADD_URL")

--set other window attributes
window~pipeFile = pipeFile
window~helpFile = helpFile
window~streaming = .false
window~radioName = window~nameEntry~get_text()
window~radioURL = window~urlEntry~get_text()
window~streamTitle = streamLabel~get_label

-- set all needed user_data
quitButton~user_data = window
helpButton~user_data = window
deleteButton~user_data = window
playButton~user_data = window
pauseButton~user_data = window
editButton~user_data = window

--connect the wanted signals to the widgets
window~signal_connect("destroy")
quitButton~signal_connect("clicked")
aboutButton~signal_connect("clicked")
helpButton~signal_connect("clicked")
playButton~signal_connect("clicked")
pauseButton~signal_connect("clicked")
addButton~signal_connect("clicked")
editButton~signal_connect("clicked")
deleteButton~signal_connect("clicked")
window~stationView~signal_connect("row_activated")

-- for the dialog buttons
--addOkButton~signal_connect("clicked")
--addCancelButton~signal_connect("clicked")

-- display everything
window~show_all()
-- do the GTK interaction loop
call gtk_main
exit

::requires 'oorexxgtk3.cls'

/*===========================================================================*/
/* mainWindow                                                                */
/*===========================================================================*/
::class MainWindow subclass GtkWindow

::constant COL_NAME 0
::constant COL_URL 1
::constant COLUMNS 2
--objects
::attribute radioDialog
::attribute nameEntry
::attribute urlEntry
::attribute selection
::attribute radioStations
::attribute stationView
::attribute listStore
::attribute streamLabel
-- strings
::attribute pipeFile
::attribute radioFile
::attribute helpFile
::attribute radioName
::attribute radioURL
::attribute streamTicker
::attribute streamTitle
::attribute streaming

::method signal_destroy
-- we need to cancel the streamTicker or this application will still hang around
  self~streamTicker~cancel
-- we also need to stop the music player
  address system "echo QUIT >>"self~pipeFile
  --call lineOut self~pipeFile,"QUIT"
  call gtk_main_quit
return

/*===========================================================================*/
/* streamScanner                                                             */
/*===========================================================================*/

::class streamScanner subclass AlarmNotification
::method triggered
--trace i
  use strict arg scanner
  window = scanner~attachment
  if window~streaming then do
    title = get_title_from_stream(window)
    window~streamTitle = title
    window~streamLabel~set_label(title~left(50))
  --say title
  end
exit
 
/*===========================================================================*/
/* stationView                                                               */
/*===========================================================================*/

::class stationView subclass GtkTreeView
::method 'signal_row_activated'
--trace i
  say self 'row_activated'
  window = self~user_data
  currentRadio = window~radioName
  --say currentRadio
  treePath = .GtkTreePath~new(arg(1)) 
  model = self~get_model
  iter = model~get_iter(treePath)
  window~radioName = model~get_value(iter,0)
  window~radioURL = model~get_value(iter,1)
  --say window~radioName
  say '"'window~radioURL'"'
  address system "echo LOAD" window~radioURL ">>"window~pipeFile
  --lineOut(window~pipeFile,"LOAD" window~radioURL)
  window~streaming = .true
  window~streamTitle = get_title_from_stream(window)
  --say window~streamTitle
  window~streamLabel~set_label(window~streamTitle~left(50))
return .true

/*===========================================================================*/
/* quitButton                                                               */
/*===========================================================================*/
::class quitButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
  window = self~user_data
-- we need to stop the streamTicker
  window~streamTicker~cancel
-- and stop the music player
  address system "echo QUIT >>"window~pipeFile  
  --call lineOut window~pipeFile,"QUIT"
  call gtk_main_quit
return .true 

/*===========================================================================*/
/* aboutButton                                                               */
/*===========================================================================*/
::class aboutButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
  dialog = .GtkAboutDialog~new()
  logo = 'img/radio.png'
  dialog~set_logo(logo)
  dialog~set_name('RexxGTK Internet Radio')
  dialog~set_version('1.0.0')
  dialog~set_copyright('(c) None, no rights reserved')
  dialog~set_comments('Listen to internet radio stations.')
  dialog~set_license('No copyright, no licence, no guarantees or warrantees, be it' || .string~nl ,
                     'explicit, implicit or whatever. Usage is totally and completely' || .string~nl ,
                     'at the users own risk, the author shall not be liable for any' || .string~nl ,
                     'damages whatsoever for any reason whatsoever.')
  dialog~set_website('http://www.oorexx.org/')
  dialog~set_website_label('ooRexx Web Site')
  dialog~set_authors('W. David Ashley - The original RexxGTK effort', ,
                     'Peter van Eerden - The program design for his gtk-server', ,
                     'Ruurd J. Idenburg - The adaptation to RexxGTK') 
  dialog~show_all()
  dialog~dialog_run()
  dialog~destroy()
return .true 

/*===========================================================================*/
/* helpButton                                                                */
/*===========================================================================*/
::class helpButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
  window = self~user_data
  call 'helpInfo.rex' window~helpFile
return .true 

/*===========================================================================*/
/* playButton                                                               */
/*===========================================================================*/
::class playButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
  window = self~user_data
  if \window~streaming then do 
    window~streaming = .true
    address system "echo PAUSE >>"window~pipeFile
  end
return .true 

/*===========================================================================*/
/* pauseButton                                                               */
/*===========================================================================*/
::class pauseButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
  window = self~user_data
  if window~streaming then  do
    window~streaming = .false
    address system "echo PAUSE >>"window~pipeFile
  end
return .true

/*===========================================================================*/
/* addButton                                                               */
/*===========================================================================*/
::class addButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
  -- user_data should be the main window
  window = self~user_data
-- blank both addDialog entry fields
  window~nameEntry~set_text("")
  window~urlEntry~set_text("")
  dlg = window~radioDialog
  dlg~show_all
  returncode = dlg~dialog_run() 
  if returnCode=.gtk~GTK_RESPONSE_OK then do
    say 'Ok' returnCode
    name = window~nameEntry~get_text()
    URL = window~urlEntry~get_text()
    window~radioStations~append(name';'URL)
    call save_stations window
    call load_stations window
    call add_stations_to_list window
    window~radioName = name
    window~radioURL = URL
  end
  if returncode=.gtk~GTK_RESPONSE_CANCEL then do
    say 'Cancel' returnCode
  end
  dlg~set_visible(.false) 
return .true

/*===========================================================================*/
/* editButton                                                               */
/*===========================================================================*/
::class editButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
  window = self~user_data
  nameEntry = window~nameEntry
  urlEntry = window~urlEntry
  nameEntry~set_text(window~radioName)
  urlEntry~set_text(window~radioURL)
  --dlg~set_title("Edit Radio Dialog")
  dlg = window~radioDialog
  dlg~show_all
  returncode = dlg~dialog_run() 
-- dialog responded with OK
  if returnCode=.gtk~GTK_RESPONSE_OK then do
    say 'Ok' returnCode
    currentName = window~radioName
    newName = nameEntry~get_text()~strip
    currentURL = window~radioURL
    newURL = urlEntry~get_text()
    if newName<>currentName | newURL<>currentURL then do
--delete current
      loop i=1 to window~radioStations~items
        parse value window~radioStations[i] with stationName ";" url 
        if currentName=stationName then do
          window~radioStations~delete(i)
         leave
        end
      end
--add new
     window~radioStations~append(newName';'newURL)
     call save_stations window
     call load_stations window
     call add_stations_to_list window
    end
    window~radioName = newName
    window~radioURL = newURL
  end
-- dialog responded with cancel, nothing has to changed
  if returncode=.gtk~GTK_RESPONSE_CANCEL then do
    say 'Cancel' returnCode
  end
  dlg~set_visible(.false)
return .true 

/*===========================================================================*/
/* deleteButton                                                               */
/*===========================================================================*/
::class deleteButton subclass GtkButton
::method signal_clicked
  say self 'clicked'
--trace i
  window = self~user_data
  view = window~stationView
  selection = window~selection
  model = view~get_model
  iter = model~get_iter_from_string("0")
  flag = selection~get_selected(model,iter)
  if flag then do
    name = model~get_value(iter,0)
    loop i=1 to window~radioStations~items
      parse value window~radioStations[i] with stationName ";" url 
      if name=stationName then do
        window~radioStations~delete(i)
        leave
      end
    end
    call save_stations window
    call load_stations window
    call add_stations_to_list window
  end
return .true 

/*===========================================================================*/
/* addOkButton                                                                */
/*===========================================================================*/

::class addOkButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
return .true

/*===========================================================================*/
/* addCancelButton                                                                */
/*===========================================================================*/

::class addCancelButton subclass GtkButton
::method signal_clicked
--trace i
  say self 'clicked'
return .true

/*===========================================================================*/
/* setup_tree_view                                                           */
/*===========================================================================*/

::routine setup_tree_view
  use strict arg treeview, COL_NAME, COL_URL

  renderer = .GtkCellRendererText~new()
  column = .GtkTreeViewColumn~newWithAttributes('Name', renderer, 'text', COL_NAME)
  treeview~append_column(column)

  renderer = .GtkCellRendererText~new()
  column = .GtkTreeViewColumn~newWithAttributes('URL', renderer, 'text', COL_URL)
  column~set_visible(.false)
  treeview~append_column(column)
  treeview~set_activate_on_single_click(1)
return

/*===========================================================================*/
/* load_stations                                                             */
/*===========================================================================*/

-- setup of radiostations known from previous sessions
-- or if none, setup the ones from the ::resource directive
-- saved stations are stored in the directory specified 
-- at the beginning as file:"radio.cfg"
::routine load_stations
--trace i
  use strict arg window
  radioConfig = window~radioFile
  if radioConfig<>.nil then do
    radioStream = .Stream~new(radioConfig)~~open()
    if radioStream~lines<>0 then do 
      radioStations = radioStream~arrayIn()
      radioStations~sort
      window~radioStations = radioStations
    end 
    else do -- if no stations yet set some
      radioStations = .resources[radio_stations]
      window~radioStations = radioStations
      call save_stations window
    end
  end
return

/*===========================================================================*/
/* add_stations_to_list                                                      */
/*===========================================================================*/

-- Add the stations from the array to the list
::routine add_stations_to_list
--trace i
  use strict arg window
  listStore = window~listStore
  listStore~clear
  do station over window~radioStations
    parse var station name ';' url
    iter = listStore~append()
    listStore~set_value(iter, window~col_name, name, window~col_url, url)
  end
return

/*===========================================================================*/
/* save_stations                                                             */
/*===========================================================================*/

::routine save_stations
  use strict arg window
--trace i
-- get the .radio.cfg file
   radioFile = window~radioFile 
-- all known stations are kept in the mainWwindow .Array attribute "radioStations"
  stations = window~radioStations
-- open the .radio.cfg file wih the "Replace option to overwrite  an existing file
  radioStream = .Stream~new(radioFile)~~open('Replace')
-- write the stations array to the file
  radioStream~arrayOut(stations) 
-- and close the file
  radioStream~close
return

/*===========================================================================*/
/* check_executable_available                                                */
/*===========================================================================*/

::routine check_executable_available
  use strict arg executable
  address system "which" executable with output stem res.
-- if not found res.0 will be zero
  if res.0=0 then do
-- display a modal error message
    flags = .gtk~GTK_DIALOG_MODAL
    type = .gtk~GTK_MESSAGE_ERROR
    buttons = .gtk~GTK_BUTTONS_CLOSE
    msg = "Can not locate "executable". Please install it, e.g. (sudo) apt install "executable
    errorDialog = .GtkMessageDialog~new(.nil,flags,type,buttons,msg)
    errorDialog~dialog_run
    errorDialog~destroy
    return .false
  end
return .true

/*===========================================================================*/
/* get_title_from_stream                                                     */
/*===========================================================================*/
/**
 * We are scanning the currently streaming station for the title of the music piece.
 * The title may (or may not) be found within the stream because of the header we give.
 * If the streaming station suport the metadata within the stream, then it normally is
 * inserted within any 65535 characters, so we scan for that amount of bytes. We catch
 * printable characters within the stream with the 'strings' command and finally see
 * if the streamtitle is found within the strings-found lines with the 'grep' command.
 *
 * 
 **/
::routine get_title_from_stream
  use strict arg window
--when only do this when we are really streaming 
  if window~streaming then do
-- timeout 1 second to establish comnnection with the url
    to = "--timeout=1"
-- if unsuccessfull just 1 retry
    tr = "--tries=1"
-- we are interested in the metadata witin the stream
    hdr = '--header="Icy-Metadata: 1"'
-- the address of the streaming radio station
    wa = window~radioURL
-- output to stdin stderr to nowhere
    out = "-O - 2>/dev/null"
-- the pipe to filter the the title of the piece of music being streamed
    pipe = "| head -c 65535 | strings | grep 'StreamTitle='" 
    address system "wget" to tr hdr wa out pipe with output stem res.
    if res.0 = 0 
      then title = wa
      else parse var res.1 . "='" title "';"
  end
return title

::routine start_music_player
  use strict arg pipe, player
  if player="mpg123" then do
    address system "rm" pipe "2>/dev/null"
    address system "mpg123 -o pulse,alsa,openal -R --fifo" pipe ">/dev/null 2>&1 &"
    address bash "while [ ! -p" pipe "]; do continue; done"
    return .true
  end
  else do
-- display a modal error message
    flags = .gtk~GTK_DIALOG_MODAL
    type = .gtk~GTK_MESSAGE_ERROR
    buttons = .gtk~GTK_BUTTONS_CLOSE
    msg = "No support available for '"player"'! Exiting ....."
    errorDialog = .GtkMessageDialog~new(.nil,flags,type,buttons,msg)
    errorDialog~dialog_run
    errorDialog~destroy
    return .false
  end

::resource radio_stations
BBC World Service;http://utulsa.streamguys1.com/KWGSHD3-MP3
Deutschlandfunk;http://st01.sslstream.dlf.de/dlf/01/128/mp3/stream.mp3
MotherEarth Radio - Classical;http://motherearth.streamserver24.com:18910/motherearth.klassik.mp3
MotherEarth Radio - Jazz;http://motherearth.streamserver24.com/listen/motherearth_jazz/motherearth.jazz.mp3
NPO Radio 2 Popular;http://icecast.omroep.nl:80/radio2-bb-mp3
NPO Radio 4 Classical;http://icecast.omroep.nl:80/radio4-bb-mp3
NPO Radio 6 Jazz & Soul;http://icecast.omroep.nl:80/radio6-bb-mp3
NPR News and Talk;http://npr-ice.streamguys1.com/live.mp3
Radio Caroline;http://sc6.radiocaroline.net:8040/mp3
Radio Paradise Mellow;http://stream.radioparadise.com/mellow-192
Radio Paradise Rock;http://stream.radioparadise.com/rock-192
Royal Concertgebouw Orchestra;http://ice.cr5.streamzilla.xlcdn.com:8000/sz=RCOLiveWebradio=mp3-192
::END    
