#!/usr/bin/env rexx
/*
 *
 */
::CLASS itemList public subclass GtkWindow
-- Class activation defines the "signal_destroy" method for it's instances.
-- And also starts the GTK interaction loop
::METHOD activate CLASS
  self~define("signal_destroy","call gtk_main_quit")
  call gtk_init
  
::ATTRIBUTE scrolledWindow
::ATTRIBUTE treeView
::ATTRIBUTE listStore
::ATTRIBUTE delimiter

::METHOD init
  use strict arg title="A GtkWinodow via RexxGTK", delimiter, width=600, height=400
-- Init method connects the instance to the "destroy" signal
  self~init:super(.gtk~GTK_WINDOW_TOPLEVEL)
  self~signal_connect("destroy")
-- specify title. size and position
  self~set_title(title)  
  self~set_size_request(width,height)
  self~delimiter = delimiter
  --self~set_position(.gtk~GTK_WIN_POS_CENTER)
-- Leave some white space border
  self~set_border_width(10)
-- Create a Scrolled Window that will contain the TextView
  scrolledWindow = .GtkScrolledWindow~new
-- Set the horizontal and vertical scrolling to AUTOMATIC
  scrolledWindow~set_policy(.gtk~GTK_POLICY_AUTOMATIC, .gtk~GTK_POLICY_AUTOMATIC)
  self~scrolledWindow = scrolledWindow
-- Create a read only GtkTreeView window with 2 columns
  self~treeView = .GtkTreeView~new
  call setupTreeView self~treeView
-- Add the TreeView to the ScrolledWindow
  self~scrolledWindow~add(self~treeView)
-- Add the ScrolledWindow to the MainWindow
  self~add(self~scrolledWindow)
    
::METHOD setText
  use strict arg text
-- text is an array of lines, delimiter split the line in prefix and content
-- Create the GtkTreeView model with 2 string columns
  listStore = .GtkListStore~new(.gtk~G_TYPE_STRING, .gtk~G_TYPE_STRING)
  do line=1 to text~items
    parse value text[line] with key (self~delimiter) content
    iter = listStore~append()
    listStore~set_value(iter, 0, key~strip(B), 1, content~strip(B))
  end
  self~treeView~set_model(listStore)        -- Add the model the GtkTreeView
exit  
  
::METHOD display
  self~show_all
  call gtk_main 
exit 
  
::ROUTINE setupTreeView
  use strict arg treeView                          -- TreeView with 2 columns
  treeView~set_headers_visible(.false)             -- Do not show the column names
  selection = treeView~get_selection               -- We dont allow row selection
  selection~set_mode(.gtk~GTK_SELECTION_NONE)
-- Create the GtkCellRenderers for the 2 columns
  renderer = .GtkCellRendererText~new()            -- text renderer for first column
                                                   -- specification for the first column
  column = .GtkTreeViewColumn~newWithAttributes('.', renderer, 'markup', 0)
  treeView~append_column(column)                   -- and add the column the the TreeView
  renderer = .GtkCellRendererText~new()            -- text renderer for column two
  renderer~set_wrap_width(300)                     -- if the text is too long to fit within the
  renderer~set_wrap_mode(.gtk~PANGO_WRAP_WORD)     -- column width, break the line at word boundary
                                                   -- specification for column two
  column = .GtkTreeViewColumn~newWithAttributes('Info', renderer, 'markup', 1)
  treeView~append_column(column)                   -- and add the column to the TreeView
return

::REQUIRES 'oorexxgtk3.cls'
